<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:oxygen="http://www.oxygenxml.com/functions"
    xmlns:File="java:java.io.File"
    exclude-result-prefixes="xs oxygen File"
    version="2.0">
    
    <xsl:template name="generateLocalizationJavascript">
        <xsl:variable name="outputFile" 
            select="concat(File:toURI(File:new(string($OUTPUTDIR))), 'assets/localization/', $LANGUAGE, '.js')"/>
        <xsl:result-document href="{$outputFile}" method="text">
            <xsl:text>
var localization = new Array();

localization["Content"]="</xsl:text>
            <xsl:value-of select="oxygen:getLocalization('Content')"/>
            <xsl:text>";
localization["Search"]="</xsl:text>
            <xsl:value-of select="oxygen:getLocalization('Search')"/>
            <xsl:text>";
localization["Search no results"]="</xsl:text>
            <xsl:value-of select="oxygen:getLocalization('Search no results')"/>
            <xsl:text>";</xsl:text>
        </xsl:result-document>
    </xsl:template>
    
    
    <xsl:function name="oxygen:getLocalization" as="xs:string">
        <xsl:param name="key"/>
        <xsl:variable name="stringsFileUri" select="concat('../../../xsl/common/strings-', $LANGUAGE, '.xml')"/>
        <xsl:variable name="allStrings" select="document($stringsFileUri)"/>
        <xsl:variable name="localizedString" select="$allStrings/strings/str[@name = $key]"/>
        <xsl:choose>
            <xsl:when test="string-length($localizedString) > 0">
                <xsl:value-of select="$localizedString"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:variable name="defaultStringsFileUri" select="'../../../xsl/common/strings-en-us.xml'"/>
                <xsl:variable name="defaultStrings" select="document($defaultStringsFileUri)"/>
                <xsl:value-of select="$defaultStrings/strings/str[@name = $key]"/>
                <xsl:message>Warning: string "<xsl:value-of select="$key"/>" not localized in <xsl:value-of 
                    select="$LANGUAGE"/> language. Using the English version: "<xsl:value-of 
                        select="$defaultStrings/strings/str[@name = $key]"/>"</xsl:message>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:function>
</xsl:stylesheet>