<?xml version="1.0" encoding="UTF-8"?>
<!-- 
  Copyright 2001-2011 Syncro Soft SRL. All rights reserved.
 -->
 <xsl:stylesheet version="2.0" 
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xhtml="http://www.w3.org/1999/xhtml"
                xmlns:oxygen="http://www.oxygenxml.com/functions"                
                exclude-result-prefixes="xsl xhtml oxygen">

  <xsl:output method="xml" indent="no" omit-xml-declaration="yes"/>
  
  <xsl:param name="folderOfPasteTargetXml"/>
  
  <!-- Main block-level conversions -->
  <xsl:template match="xhtml:html">
    <xsl:apply-templates select="xhtml:body" mode="checkHeadings"/>
  </xsl:template>
  
  <!-- 
      MsoTitle - attr value for MS Word titles. 
      In XHTML that comes from MS Word the last node is a comment with the content 'EndFragment'.
  -->
  <xsl:template match="xhtml:div[xhtml:p[@class = 'MsoTitle']]">
    <xsl:variable name="sentinel" select="
      ( following-sibling::xhtml:div[xhtml:p[@class = 'MsoTitle']]
      | following-sibling::comment()[. = 'EndFragment'])[1]"/>
    <xsl:variable name="firstChild" select="
      (following-sibling::xhtml:h1,
       following-sibling::xhtml:h2,
       following-sibling::xhtml:h3,
       following-sibling::xhtml:h4,
       following-sibling::xhtml:h5,
       following-sibling::xhtml:h6)
       [. &lt;&lt; $sentinel][1]"/>
    <xsl:variable name="title" select="xhtml:p[@class = 'MsoTitle']//text()"/>
    <xsl:call-template name="fillSectionBody">
      <xsl:with-param name="title" select="$title"/>
      <xsl:with-param name="firstChild" select="$firstChild"/>
      <xsl:with-param name="sentinel" select="$sentinel"/>
      <xsl:with-param name="bodySet" select="following-sibling::text() | following-sibling::xhtml:*"/>
    </xsl:call-template>
  </xsl:template>
  
  <xsl:template name="fillSectionBody">
    <xsl:param name="title"/>
    <xsl:param name="firstChild"/>
    <xsl:param name="sentinel"/>
    <xsl:param name="bodySet"/>
    <xsl:variable name="body">
      <xsl:choose>
        <xsl:when test="$firstChild">
          <xsl:apply-templates select="$bodySet[. &lt;&lt; $firstChild]" mode="preprocess"/>
        </xsl:when>
        <xsl:when test="$sentinel">
          <xsl:apply-templates select="$bodySet[. &lt;&lt; $sentinel]" mode="preprocess"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:choose>
            <xsl:when test="some $x in $bodySet satisfies 
                local-name($x) = 'h1' and namespace-uri($x) = 'http://www.w3.org/1999/xhtml'">
              <xsl:variable name="limit" select="(for $x in $bodySet return $x 
                [local-name() = 'h1'][namespace-uri() = 'http://www.w3.org/1999/xhtml'])[1]">
              </xsl:variable>
              <xsl:apply-templates select="$bodySet[. &lt;&lt; $limit]" mode="preprocess"/>
            </xsl:when>
            <xsl:when test="some $x in $bodySet satisfies 
              local-name($x) = 'h2' and namespace-uri($x) = 'http://www.w3.org/1999/xhtml'">
              <xsl:variable name="limit" select="(for $x in $bodySet return $x 
                [local-name() = 'h2'][namespace-uri() = 'http://www.w3.org/1999/xhtml'])[1]"/>
              <xsl:apply-templates select="$bodySet[. &lt;&lt; $limit]" mode="preprocess"/>
            </xsl:when>
            <xsl:when test="some $x in $bodySet satisfies 
              local-name($x) = 'h3' and namespace-uri($x) = 'http://www.w3.org/1999/xhtml'">
              <xsl:variable name="limit" select="(for $x in $bodySet return $x 
                [local-name() = 'h3'][namespace-uri() = 'http://www.w3.org/1999/xhtml'])[1]"/>
              <xsl:apply-templates select="$bodySet[. &lt;&lt; $limit]" mode="preprocess"/>
            </xsl:when>
            <xsl:when test="some $x in $bodySet satisfies 
              local-name($x) = 'h4' and namespace-uri($x) = 'http://www.w3.org/1999/xhtml'">
              <xsl:variable name="limit" select="(for $x in $bodySet return $x 
                [local-name() = 'h4'][namespace-uri() = 'http://www.w3.org/1999/xhtml'])[1]"/>
              <xsl:apply-templates select="$bodySet[. &lt;&lt; $limit]" mode="preprocess"/>
            </xsl:when>
            <xsl:when test="some $x in $bodySet satisfies 
              local-name($x) = 'h5' and namespace-uri($x) = 'http://www.w3.org/1999/xhtml'">
              <xsl:variable name="limit" select="(for $x in $bodySet return $x 
                [local-name() = 'h5'][namespace-uri() = 'http://www.w3.org/1999/xhtml'])[1]"/>
              <xsl:apply-templates select="$bodySet[. &lt;&lt; $limit]" mode="preprocess"/>
            </xsl:when>
            <xsl:when test="some $x in $bodySet satisfies 
              local-name($x) = 'h6' and namespace-uri($x) = 'http://www.w3.org/1999/xhtml'">
              <xsl:variable name="limit" select="(for $x in $bodySet return $x 
                [local-name() = 'h6'][namespace-uri() = 'http://www.w3.org/1999/xhtml'])[1]"/>
              <xsl:apply-templates select="$bodySet[. &lt;&lt; $limit]" mode="preprocess"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:apply-templates select="$bodySet"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="afterBody">
      <xsl:choose>
        <xsl:when test="$sentinel">
          <xsl:choose>
            <xsl:when test="some $x in $bodySet satisfies 
              local-name($x) = 'h1' and 
              namespace-uri($x) = 'http://www.w3.org/1999/xhtml' and
              $x &lt;&lt; $sentinel">
              <xsl:apply-templates select="for $x in $bodySet return $x 
                [local-name($x) = 'h1']
                [namespace-uri($x) = 'http://www.w3.org/1999/xhtml']
                [$x &lt;&lt; $sentinel]"/>
            </xsl:when>
            <xsl:when test="some $x in $bodySet satisfies 
              local-name($x) = 'h2' and 
              namespace-uri($x) = 'http://www.w3.org/1999/xhtml' and
              $x &lt;&lt; $sentinel">
              <xsl:apply-templates select="for $x in $bodySet return $x 
                [local-name($x) = 'h2']
                [namespace-uri($x) = 'http://www.w3.org/1999/xhtml']
                [$x &lt;&lt; $sentinel]"/>
            </xsl:when>
            <xsl:when test="some $x in $bodySet satisfies 
              local-name($x) = 'h3' and 
              namespace-uri($x) = 'http://www.w3.org/1999/xhtml' and
              $x &lt;&lt; $sentinel">
              <xsl:apply-templates select="for $x in $bodySet return $x 
                [local-name($x) = 'h3']
                [namespace-uri($x) = 'http://www.w3.org/1999/xhtml']
                [$x &lt;&lt; $sentinel]"/>
            </xsl:when>
            <xsl:when test="some $x in $bodySet satisfies 
              local-name($x) = 'h4' and 
              namespace-uri($x) = 'http://www.w3.org/1999/xhtml' and
              $x &lt;&lt; $sentinel">
              <xsl:apply-templates select="for $x in $bodySet return $x 
                [local-name($x) = 'h4']
                [namespace-uri($x) = 'http://www.w3.org/1999/xhtml']
                [$x &lt;&lt; $sentinel]"/>
            </xsl:when>
            <xsl:when test="some $x in $bodySet satisfies 
              local-name($x) = 'h5' and 
              namespace-uri($x) = 'http://www.w3.org/1999/xhtml' and
              $x &lt;&lt; $sentinel">
              <xsl:apply-templates select="for $x in $bodySet return $x 
                [local-name($x) = 'h5']
                [namespace-uri($x) = 'http://www.w3.org/1999/xhtml']
                [$x &lt;&lt; $sentinel]"/>
            </xsl:when>
            <xsl:when test="some $x in $bodySet satisfies 
              local-name($x) = 'h6' and 
              namespace-uri($x) = 'http://www.w3.org/1999/xhtml' and
              $x &lt;&lt; $sentinel">
              <xsl:apply-templates select="for $x in $bodySet return $x 
                [local-name($x) = 'h6']
                [namespace-uri($x) = 'http://www.w3.org/1999/xhtml']
                [$x &lt;&lt; $sentinel]"/>
            </xsl:when>
          </xsl:choose>
        </xsl:when>
        <xsl:otherwise>
          <xsl:choose>
            <xsl:when test="some $x in $bodySet satisfies 
              local-name($x) = 'h1' and 
              namespace-uri($x) = 'http://www.w3.org/1999/xhtml'">
              <xsl:apply-templates select="for $x in $bodySet return $x 
                [local-name($x) = 'h1']
                [namespace-uri($x) = 'http://www.w3.org/1999/xhtml']"/>
            </xsl:when>
            <xsl:when test="some $x in $bodySet satisfies 
              local-name($x) = 'h2' and 
              namespace-uri($x) = 'http://www.w3.org/1999/xhtml'">
              <xsl:apply-templates select="for $x in $bodySet return $x 
                [local-name($x) = 'h2']
                [namespace-uri($x) = 'http://www.w3.org/1999/xhtml']"/>
            </xsl:when>
            <xsl:when test="some $x in $bodySet satisfies 
              local-name($x) = 'h3' and 
              namespace-uri($x) = 'http://www.w3.org/1999/xhtml'">
              <xsl:apply-templates select="for $x in $bodySet return $x 
                [local-name($x) = 'h3']
                [namespace-uri($x) = 'http://www.w3.org/1999/xhtml']"/>
            </xsl:when>
            <xsl:when test="some $x in $bodySet satisfies 
              local-name($x) = 'h4' and 
              namespace-uri($x) = 'http://www.w3.org/1999/xhtml'">
              <xsl:apply-templates select="for $x in $bodySet return $x 
                [local-name($x) = 'h4']
                [namespace-uri($x) = 'http://www.w3.org/1999/xhtml']"/>
            </xsl:when>
            <xsl:when test="some $x in $bodySet satisfies 
              local-name($x) = 'h5' and 
              namespace-uri($x) = 'http://www.w3.org/1999/xhtml'">
              <xsl:apply-templates select="for $x in $bodySet return $x 
                [local-name($x) = 'h5']
                [namespace-uri($x) = 'http://www.w3.org/1999/xhtml']"/>
            </xsl:when>
            <xsl:when test="some $x in $bodySet satisfies 
              local-name($x) = 'h6' and 
              namespace-uri($x) = 'http://www.w3.org/1999/xhtml'">
              <xsl:apply-templates select="for $x in $bodySet return $x 
                [local-name($x) = 'h6']
                [namespace-uri($x) = 'http://www.w3.org/1999/xhtml']"/>
            </xsl:when>
          </xsl:choose>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:choose>
      <xsl:when test="ancestor::xhtml:td | ancestor::xhtml:th">
        <xsl:copy-of select="$title"/>
        <xsl:copy-of select="$body"/>
        <xsl:copy-of select="$afterBody"/>
      </xsl:when>
      <xsl:otherwise>
        <topic>
          <title>
            <xsl:copy-of select="$title"/>
          </title>
          <body>
            <xsl:copy-of select="$body"/>
          </body>
          <xsl:copy-of select="$afterBody"/>
        </topic>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="xhtml:h1">
    <xsl:variable name="sentinel" select="
      ( following-sibling::xhtml:div[xhtml:p[@class = 'MsoTitle']]
      | following-sibling::xhtml:h1
      | following-sibling::comment()[. = 'EndFragment'])[1]"/>
    <xsl:variable name="firstChild" select="
      (following-sibling::xhtml:h2, 
       following-sibling::xhtml:h3,
       following-sibling::xhtml:h4,
       following-sibling::xhtml:h5,
       following-sibling::xhtml:h6)
      [. &lt;&lt; $sentinel][1]"/>
    <xsl:variable name="title">
      <xsl:apply-templates/>
    </xsl:variable>
    <xsl:call-template name="fillSectionBody">
      <xsl:with-param name="title" select="$title"/>
      <xsl:with-param name="firstChild" select="$firstChild"/>
      <xsl:with-param name="sentinel" select="$sentinel"/>
      <xsl:with-param name="bodySet" select="following-sibling::text() | following-sibling::xhtml:*"/>
    </xsl:call-template>
  </xsl:template>
  
  <xsl:template match="xhtml:h2">
    <xsl:variable name="sentinel" select="
      ( following-sibling::xhtml:div[xhtml:p[@class = 'MsoTitle']]
      | following-sibling::xhtml:h1
      | following-sibling::xhtml:h2
      | following-sibling::comment()[. = 'EndFragment'])[1]"/>
    <xsl:variable name="firstChild" select="
      (following-sibling::xhtml:h3,
       following-sibling::xhtml:h4,
       following-sibling::xhtml:h5,
       following-sibling::xhtml:h6)
       [. &lt;&lt; $sentinel][1]"/>
    <xsl:variable name="title">
      <xsl:apply-templates/>
    </xsl:variable>
    <xsl:call-template name="fillSectionBody">
      <xsl:with-param name="title" select="$title"/>
      <xsl:with-param name="firstChild" select="$firstChild"/>
      <xsl:with-param name="sentinel" select="$sentinel"/>
      <xsl:with-param name="bodySet" select="following-sibling::text() | following-sibling::xhtml:*"/>
    </xsl:call-template>
  </xsl:template>
  
  <xsl:template match="xhtml:h3">
    <xsl:variable name="sentinel" select="
      ( following-sibling::xhtml:div[xhtml:p[@class = 'MsoTitle']]
      | following-sibling::xhtml:h1
      | following-sibling::xhtml:h2
      | following-sibling::xhtml:h3
      | following-sibling::comment()[. = 'EndFragment'])[1]"/>
    <xsl:variable name="firstChild" select="
      (following-sibling::xhtml:h4,
       following-sibling::xhtml:h5,
       following-sibling::xhtml:h6)
       [. &lt;&lt; $sentinel][1]"/>
    <xsl:variable name="title">
      <xsl:apply-templates/>
    </xsl:variable>
    <xsl:call-template name="fillSectionBody">
      <xsl:with-param name="title" select="$title"/>
      <xsl:with-param name="firstChild" select="$firstChild"/>
      <xsl:with-param name="sentinel" select="$sentinel"/>
      <xsl:with-param name="bodySet" select="following-sibling::text() | following-sibling::xhtml:*"/>
    </xsl:call-template>
  </xsl:template>
  
  <xsl:template match="xhtml:h4">
    <xsl:variable name="sentinel" select="
      ( following-sibling::xhtml:div[xhtml:p[@class = 'MsoTitle']]
      | following-sibling::xhtml:h1
      | following-sibling::xhtml:h2
      | following-sibling::xhtml:h3
      | following-sibling::xhtml:h4
      | following-sibling::comment()[. = 'EndFragment'])[1]"/>
    <xsl:variable name="firstChild" select="
      (following-sibling::xhtml:h5,
       following-sibling::xhtml:h6)
       [. &lt;&lt; $sentinel][1]"/>
    <xsl:variable name="title">
      <xsl:apply-templates/>
    </xsl:variable>
    <xsl:call-template name="fillSectionBody">
      <xsl:with-param name="title" select="$title"/>
      <xsl:with-param name="firstChild" select="$firstChild"/>
      <xsl:with-param name="sentinel" select="$sentinel"/>
      <xsl:with-param name="bodySet" select="following-sibling::text() | following-sibling::xhtml:*"/>
    </xsl:call-template>
  </xsl:template>
  
  <xsl:template match="xhtml:h5">
    <xsl:variable name="sentinel" select="
      ( following-sibling::xhtml:div[xhtml:p[@class = 'MsoTitle']]
      | following-sibling::xhtml:h1
      | following-sibling::xhtml:h2
      | following-sibling::xhtml:h3
      | following-sibling::xhtml:h4
      | following-sibling::xhtml:h5
      | following-sibling::comment()[. = 'EndFragment'])[1]"/>
    <xsl:variable name="firstChild" select="following-sibling::xhtml:h6[. &lt;&lt; $sentinel][1]"/>
    <xsl:variable name="title">
      <xsl:apply-templates/>
    </xsl:variable>
    <xsl:call-template name="fillSectionBody">
      <xsl:with-param name="title" select="$title"/>
      <xsl:with-param name="firstChild" select="$firstChild"/>
      <xsl:with-param name="sentinel" select="$sentinel"/>
      <xsl:with-param name="bodySet" select="following-sibling::text() | following-sibling::xhtml:*"/>
    </xsl:call-template>
  </xsl:template>
  
  <xsl:template match="xhtml:h6">
    <xsl:variable name="sentinel" select="
      ( following-sibling::xhtml:div[xhtml:p[@class = 'MsoTitle']]
      | following-sibling::xhtml:h1
      | following-sibling::xhtml:h2
      | following-sibling::xhtml:h3
      | following-sibling::xhtml:h4
      | following-sibling::xhtml:h5
      | following-sibling::xhtml:h6
      | following-sibling::comment()[. = 'EndFragment'])[1]"/>
    <xsl:variable name="title">
      <xsl:apply-templates/>
    </xsl:variable>
    <xsl:call-template name="fillSectionBody">
      <xsl:with-param name="title" select="$title"/>
      <xsl:with-param name="sentinel" select="$sentinel"/>
      <xsl:with-param name="bodySet" select="following-sibling::text() | following-sibling::xhtml:*"/>
    </xsl:call-template>
  </xsl:template>
  
  <xsl:template match="xhtml:h1[ancestor::xhtml:dl] 
                                  | xhtml:h2[ancestor::xhtml:dl] 
                                  | xhtml:h3[ancestor::xhtml:dl] 
                                  | xhtml:h4[ancestor::xhtml:dl] 
                                  | xhtml:h5[ancestor::xhtml:dl]
                                  | xhtml:h6[ancestor::xhtml:dl]">
    <b>
      <xsl:apply-templates/>
    </b>
  </xsl:template>

  <xsl:template match="xhtml:p[contains(@class, 'MsoListParagraph')]"/>
     
  <xsl:template match="xhtml:p[not(contains(@class, 'MsoListParagraph'))]">
   <xsl:choose>
     <xsl:when test="not(parent::xhtml:th | parent::xhtml:td) and not(normalize-space(.) = '')">
       <p>
         <xsl:apply-templates select="@*"/>
         <xsl:apply-templates select="." mode="checkHeadings"/>
       </p>
     </xsl:when>
     <xsl:otherwise>
       <xsl:apply-templates select="@*"/>
       <xsl:apply-templates select="." mode="checkHeadings"/>
      </xsl:otherwise>
   </xsl:choose>
  </xsl:template>

   <xsl:template match="xhtml:span[preceding-sibling::xhtml:p and not(following-sibling::*)]">
     <p>
       <xsl:apply-templates select="@*"/>
       <xsl:apply-templates select="." mode="checkHeadings"/>
     </p>
   </xsl:template>
     
   <xsl:template match="xhtml:div[xhtml:br] | xhtml:p[xhtml:br]">
    <xsl:call-template name="brContent"/>
  </xsl:template>
  
  <xsl:template name="brContent">
    <xsl:variable name="subsetContent">
      <xhtml:div>
        <xsl:copy-of select="node()[not(preceding-sibling::xhtml:br)]"/>
      </xhtml:div>
    </xsl:variable>
    <xsl:variable name="preceding-text">
      <xsl:apply-templates select="$subsetContent/xhtml:div" mode="checkHeadings"/>
    </xsl:variable>
    <xsl:if test="string-length(normalize-space($preceding-text)) > 0">
      <p><xsl:copy-of select="$preceding-text"/></p>
    </xsl:if>
    <xsl:for-each select="xhtml:br">
      <xsl:variable name="subsetContent">
        <xhtml:div>
          <xsl:copy-of select="parent::*[1]/node()[current() is preceding-sibling::xhtml:br[1]]"/>
        </xhtml:div>
      </xsl:variable>
      <xsl:variable name="following-text">
        <xsl:apply-templates select="$subsetContent/xhtml:div" mode="checkHeadings"/>
      </xsl:variable>
      <xsl:if test="string-length(normalize-space($following-text)) > 0">
        <p><xsl:copy-of select="$following-text"/></p>
      </xsl:if>
    </xsl:for-each>
  </xsl:template>
  
  <xsl:template match="xhtml:li[xhtml:br]">
    <li>
      <xsl:call-template name="brContent"/>
    </li>
  </xsl:template>
  
  <xsl:template match="xhtml:pre">
    <pre>
       <xsl:apply-templates select="@*"/>
      <xsl:apply-templates select="." mode="checkHeadings"/>
    </pre>
  </xsl:template>
  
  <!-- Hyperlinks -->
   <xsl:template match="xhtml:a[starts-with(@href, 'https://') or
     starts-with(@href,'http://') or starts-with(@href,'ftp://')]" priority="1.5">
    <xref>
      <xsl:attribute name="href">
        <xsl:value-of select="normalize-space(@href)"/>
      </xsl:attribute>
      <xsl:attribute name="format">html</xsl:attribute>
      <xsl:attribute name="scope">external</xsl:attribute>
      <xsl:apply-templates select="@*"/>
      <xsl:apply-templates select="." mode="checkHeadings"/>
   </xref>
  </xsl:template>
  
  <xsl:template match="xhtml:a[contains(@href,'#')]" priority="0.6">
    <xref>
      <xsl:attribute name="href">
        <xsl:call-template name="make_id">
          <xsl:with-param name="string" select="normalize-space(@href)"/>
        </xsl:call-template>
      </xsl:attribute>
      <xsl:apply-templates select="@*"/>
      <xsl:apply-templates select="." mode="checkHeadings"/>
    </xref>
  </xsl:template>
  
  <xsl:template match="xhtml:a[@name != '']" priority="0.6">
    <ph>
      <xsl:attribute name="id">
     <xsl:call-template name="make_id">
       <xsl:with-param name="string" select="normalize-space(@name)"/>
     </xsl:call-template>
      </xsl:attribute>
      <xsl:apply-templates select="@*"/>
      <xsl:apply-templates select="." mode="checkHeadings"/>
    </ph>
  </xsl:template>
  
  <xsl:template match="xhtml:a[@href != '']">
    <xref>
      <xsl:attribute name="href">
        <xsl:call-template name="make_id">
          <xsl:with-param name="string" select="normalize-space(@href)"/>
        </xsl:call-template>
      </xsl:attribute>
      <xsl:apply-templates select="@*"/>
      <xsl:apply-templates select="." mode="checkHeadings"/>
    </xref>
  </xsl:template>
  
  <xsl:template name="make_id">
    <xsl:param name="string" select="''"/>
    <xsl:call-template name="getFilename">
      <xsl:with-param name="path" select="translate($string,' \()','_/_')"/>
    </xsl:call-template>
  </xsl:template>
  
  <xsl:template name="string.subst">
   <xsl:param name="string" select="''"/>
   <xsl:param name="substitute" select="''"/>
   <xsl:param name="with" select="''"/>
   <xsl:choose>
    <xsl:when test="contains($string,$substitute)">
     <xsl:variable name="pre" select="substring-before($string,$substitute)"/>
     <xsl:variable name="post" select="substring-after($string,$substitute)"/>
     <xsl:call-template name="string.subst">
      <xsl:with-param name="string" select="concat($pre,$with,$post)"/>
      <xsl:with-param name="substitute" select="$substitute"/>
      <xsl:with-param name="with" select="$with"/>
     </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
     <xsl:value-of select="$string"/>
    </xsl:otherwise>
   </xsl:choose>
  </xsl:template>
  
  <!-- Images -->
  <xsl:template match="xhtml:img">
    <xsl:variable name="pastedImageURL" 
      xmlns:URL="java:java.net.URL"
      xmlns:URLUtil="java:ro.sync.util.URLUtil"
      xmlns:UUID="java:java.util.UUID">
      <xsl:choose>
        <xsl:when test="namespace-uri-for-prefix('o', .) = 'urn:schemas-microsoft-com:office:office'">
          <!-- Copy from MS Office. Copy the image from user temp folder to folder of XML document
            that is the paste target. -->
          <xsl:variable name="imageFilename">
            <xsl:variable name="fullPath" select="URL:getPath(URL:new(translate(@src, '\', '/')))"/>
            <xsl:variable name="srcFile">
              <xsl:choose>
                <xsl:when test="contains($fullPath, ':')">
                  <xsl:value-of select="substring($fullPath, 2)"/>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:value-of select="$fullPath"/>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:variable>
            <xsl:call-template name="getFilename">
              <xsl:with-param name="path" select="string($srcFile)"/>
            </xsl:call-template>
          </xsl:variable>
          <xsl:variable name="stringImageFilename" select="string($imageFilename)"/>
          <xsl:variable name="uid" select="UUID:hashCode(UUID:randomUUID())"/>
          <xsl:variable name="uniqueTargetFilename" select="concat(substring-before($stringImageFilename, '.'), '_', $uid, '.', substring-after($stringImageFilename, '.'))"/>
          <xsl:variable name="sourceURL" select="URL:new(translate(@src, '\', '/'))"/>
          <xsl:variable name="correctedSourceFile">
            <xsl:choose>
              <xsl:when test="contains(URL:getPath($sourceURL), ':')">
                <xsl:value-of select="substring-after(URL:getPath($sourceURL), '/')"/>
              </xsl:when>
              <xsl:otherwise>
                <xsl:value-of select="URL:getPath($sourceURL)"/>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:variable>
          <xsl:variable name="sourceFile" select="URLUtil:uncorrect($correctedSourceFile)"/>
          <xsl:variable name="targetURL" select="URL:new(concat($folderOfPasteTargetXml, '/', $uniqueTargetFilename))"/>
          <xsl:value-of select="URLUtil:copyURL($sourceURL, $targetURL)"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="@src"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    
    <image href="{$pastedImageURL}">
      <xsl:if test="@height != ''">
        <xsl:attribute name="height">
          <xsl:value-of select="@height"/>
        </xsl:attribute>
      </xsl:if>
      <xsl:if test="@width != ''">
        <xsl:attribute name="width">
          <xsl:value-of select="@width"/>
        </xsl:attribute>
      </xsl:if>
    </image>
  </xsl:template>
  
  <xsl:template name="getFilename">
   <xsl:param name="path"/>
   <xsl:choose>
    <xsl:when test="contains($path,'/')">
     <xsl:call-template name="getFilename">
      <xsl:with-param name="path" select="substring-after($path,'/')"/>
     </xsl:call-template>
    </xsl:when>
     <xsl:when test="contains($path,'\')">
       <xsl:call-template name="getFilename">
         <xsl:with-param name="path" select="substring-after($path,'\')"/>
       </xsl:call-template>
     </xsl:when>
     <xsl:otherwise>
     <xsl:value-of select="$path"/>
    </xsl:otherwise>
   </xsl:choose>
  </xsl:template>
  
  <!-- List elements -->
  <xsl:template match="xhtml:ul">
    <ul>
      <xsl:apply-templates select="@*"/>
      <xsl:apply-templates select="." mode="checkHeadings"/>
    </ul>
  </xsl:template>
  
  <xsl:template match="xhtml:ol">
    <ol>
      <xsl:apply-templates select="@*"/>
      <xsl:apply-templates select="." mode="checkHeadings"/>
    </ol>
  </xsl:template>
  
  
  <!-- MS Office START -->
  
  <!-- Unordered lists from MS Word can be translated as Docbook unordered lists only when they can
    be identified, that is the @style attrib contains ''. This is true for single level lists
    but never for multi-level list. If they cannot be identified as unordered lists will be
    translated as ordered lists. -->
  
  <!--
      Unordered lists in MS Word. <p> and <span> can be separated by <i> and <b>.
  -->
  <!-- List with Normal style. -->
  <xsl:template match="xhtml:p[contains(@class, 'MsoListParagraphCxSpFirst')]
                                               [descendant::xhtml:span[contains(@style, 'mso-fareast-font-family')]]" 
                       priority="1.5">    
    <ul>
      <li>
        <p>
          <xsl:value-of select=".//text()"/>
        </p>
      </li>
      <xsl:variable name="sentinel" select="following-sibling::xhtml:p[contains(@class, 'MsoListParagraphCxSpLast')]
        [descendant::xhtml:span[contains(@style, 'mso-fareast-font-family')]][1]/following-sibling::*[1]"/> 
      <xsl:for-each select="following-sibling::xhtml:p[contains(@class, 'MsoListParagraphCxSp')][. &lt;&lt; $sentinel]">
        <li>
          <p>
            <xsl:value-of select=".//text()"/>
          </p>
        </li>
      </xsl:for-each>
    </ul>
  </xsl:template>
  
   <!-- List with List Paragraph style. -->
   <xsl:template match="xhtml:p[contains(@class, 'MsoListParagraph')]
          [not(preceding-sibling::*[3][self::xhtml:p[contains(@class, 'MsoListParagraph')]])]" 
          priority="1.3">
    <xsl:variable name="condition" select="matches(substring(normalize-space(xhtml:span[1]), 1, 1), '[a-zA-Z0-9]')"></xsl:variable>
    <xsl:variable name="listContent">
      <li>
        <p>
            <xsl:value-of select="xhtml:span[2] | text()"/>
        </p>
      </li>
      <xsl:for-each select="following-sibling::xhtml:p[contains(@class, 'MsoListParagraph')]
          [preceding-sibling::*[3][self::xhtml:p[contains(@class, 'MsoListParagraph')]]]">
        <li>
          <p>
            <xsl:value-of select="xhtml:span[2] | text()"/>
          </p>
        </li>
      </xsl:for-each>
    </xsl:variable>
    <xsl:choose>
      <xsl:when test="$condition">
        <ol>
          <xsl:copy-of select="$listContent"/>
        </ol>
      </xsl:when>
      <xsl:otherwise>
        <ul>
          <xsl:copy-of select="$listContent"/>
        </ul>
      </xsl:otherwise>
    </xsl:choose>
   </xsl:template>
   
  <!-- Ordered lists from MS Word -->
  <xsl:template match="xhtml:p[contains(@class, 'MsoListParagraphCxSpFirst')]" priority="1.4">
    <ol>
      <li>
        <p>
          <xsl:value-of select=".//text()"/>
        </p>
      </li>
      <xsl:variable name="sentinel" select="following-sibling::xhtml:p[contains(@class, 'MsoListParagraphCxSpLast')][1]/following-sibling::*[1]"/>
      <xsl:for-each select="following-sibling::xhtml:p[contains(@class, 'MsoListParagraphCxSp')][. &lt;&lt; $sentinel]">
        <li>
          <p>
            <xsl:value-of select=".//text()"/>
          </p>
        </li>
      </xsl:for-each>
    </ol>
  </xsl:template>
  
  <xsl:template match="xhtml:p[contains(@class, 'MsoListParagraphCxSpMiddle') 
                                            or contains(@class, 'MsoListParagraphCxSpLast')]" priority="1.5"/>
  
  <!-- MS Office END -->  


  <xsl:template match="xhtml:kbd">
    <userinput>
      <xsl:apply-templates select="." mode="checkHeadings"/>
    </userinput>
  </xsl:template>
  
  <xsl:template match="xhtml:samp">
    <systemoutput>
      <xsl:apply-templates select="." mode="checkHeadings"/>
    </systemoutput>
  </xsl:template>
  
  <xsl:template match="xhtml:blockquote">
    <lq>
      <xsl:apply-templates select="." mode="checkHeadings"/>
    </lq>
  </xsl:template>
  
  <xsl:template match="xhtml:q">
    <q>
      <xsl:apply-templates select="." mode="checkHeadings"/>
    </q>
  </xsl:template>
  
  <xsl:template match="xhtml:dl">
    <dl>
    	<xsl:apply-templates select="@*"/>
    	<xsl:variable name="dataBeforeTitle" select="xhtml:dd[empty(preceding-sibling::xhtml:dt)]"/>
    	<xsl:if test="not(empty($dataBeforeTitle))">
    		<dlentry>
    			<dt/>
    			<xsl:for-each select="$dataBeforeTitle">
    				<xsl:apply-templates select="."/>
    			</xsl:for-each>
    		</dlentry>
    	</xsl:if>
    	<xsl:for-each select="xhtml:dt">
    		<dlentry>
    			<xsl:apply-templates select="."/>
    			<xsl:apply-templates select="following-sibling::xhtml:dd[current() is preceding-sibling::xhtml:dt[1]]"/>
    		</dlentry>
    	</xsl:for-each>
    </dl>
  </xsl:template>
  
  <xsl:template match="xhtml:dt">
    <dt>
      <xsl:apply-templates select="node()" mode="preprocess"/>
    </dt>
  </xsl:template>
  
  <xsl:template match="xhtml:dd">
    <dd>
      <xsl:apply-templates select="node()" mode="preprocess"/>
    </dd>
  </xsl:template>
  
  <xsl:template match="xhtml:li">
    <li>
      <xsl:apply-templates select="@*"/>
      <xsl:choose>
       <xsl:when test="count(xhtml:p) = 0">
         <p>
           <xsl:apply-templates select="." mode="checkHeadings"/>
         </p>
       </xsl:when>
       <xsl:otherwise>
         <xsl:apply-templates select="." mode="checkHeadings"/>
       </xsl:otherwise>
    </xsl:choose>
   </li>
  </xsl:template>
          
  <xsl:template match="*" mode="checkHeadings">
    <xsl:choose>
      <xsl:when test="xhtml:div[xhtml:p[@class = 'MsoTitle']]">
        <xsl:apply-templates select="(xhtml:* | text())[. &lt;&lt; current()/xhtml:div[xhtml:p[@class = 'MsoTitle']][1]]" mode="preprocess"/>
        <xsl:apply-templates select="xhtml:div[xhtml:p[@class = 'MsoTitle']]"/>
      </xsl:when>
      <xsl:when test="xhtml:h1">
        <xsl:apply-templates select="(xhtml:* | text())[. &lt;&lt; current()/xhtml:h1[1]]" mode="preprocess"/>
        <xsl:apply-templates select="xhtml:h1"/>
      </xsl:when>
      <xsl:when test="xhtml:h2">
        <xsl:apply-templates select="(xhtml:* | text())[. &lt;&lt; current()/xhtml:h2[1]]" mode="preprocess"/>
        <xsl:apply-templates select="xhtml:h2"/>
      </xsl:when>
      <xsl:when test="xhtml:h3">
        <xsl:apply-templates select="(xhtml:* | text())[. &lt;&lt; current()/xhtml:h3[1]]" mode="preprocess"/>
        <xsl:apply-templates select="xhtml:h3"/>
      </xsl:when>
      <xsl:when test="xhtml:h4">
        <xsl:apply-templates select="(xhtml:* | text())[. &lt;&lt; current()/xhtml:h4[1]]" mode="preprocess"/>
        <xsl:apply-templates select="xhtml:h4"/>
      </xsl:when>
      <xsl:when test="xhtml:h5">
        <xsl:apply-templates select="(xhtml:* | text())[. &lt;&lt; current()/xhtml:h5[1]]" mode="preprocess"/>
        <xsl:apply-templates select="xhtml:h5"/>
      </xsl:when>
      <xsl:when test="xhtml:h6">
        <xsl:apply-templates select="(xhtml:* | text())[. &lt;&lt; current()/xhtml:h6[1]]" mode="preprocess"/>
        <xsl:apply-templates select="xhtml:h6"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:apply-templates select="node()"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="@id" mode="#all"> 
    <xsl:attribute name="id">
      <xsl:value-of select="."/>
    </xsl:attribute>
  </xsl:template>
  
  <xsl:template match="@*" mode="#all">
    <!--<xsl:message>No template for attribute <xsl:value-of select="name()"/></xsl:message>-->
  </xsl:template>
  
  <!-- Inline formatting -->
  <xsl:template match="xhtml:b | xhtml:strong">
    <b>
      <xsl:apply-templates select="." mode="checkHeadings"/>
    </b>
  </xsl:template>
  <xsl:template match="xhtml:i | xhtml:em">
    <i>
      <xsl:apply-templates select="." mode="checkHeadings"/>
    </i>
  </xsl:template>
  <xsl:template match="xhtml:u">
    <u>
      <xsl:apply-templates select="." mode="checkHeadings"/>
    </u>
  </xsl:template>
          
  <!-- Ignored elements -->
  <xsl:template match="xhtml:hr"/>
  <!--<xsl:template match="xhtml:h1[1]|xhtml:h2[1]|xhtml:h3[1]" priority="1"/>-->
  <xsl:template match="xhtml:br"/>
  <xsl:template match="xhtml:meta"/>
  <xsl:template match="xhtml:style"/>
  <xsl:template match="xhtml:script"/>
  <xsl:template match="xhtml:p[normalize-space(.) = '' and count(*) = 0]" priority="0.6"/>
  <xsl:template match="text()" mode="#all">
   <xsl:choose>
    <xsl:when test="normalize-space(.) = ''"><xsl:text> </xsl:text></xsl:when>
    <xsl:otherwise><xsl:copy/></xsl:otherwise>
   </xsl:choose>
  </xsl:template>
  
  <xsl:template match="xhtml:a[@href != '' 
                        and not(boolean(ancestor::xhtml:p|ancestor::xhtml:li))]" 
                priority="1">
    <p>
      <xref>
        <xsl:attribute name="linkend">
         <xsl:call-template name="make_id">
           <xsl:with-param name="string" select="normalize-space(@href)"/>
         </xsl:call-template>
        </xsl:attribute>
        <xsl:apply-templates select="@*"/>
        <xsl:apply-templates select="." mode="checkHeadings"/>
      </xref>
   </p>
  </xsl:template>
  
  <xsl:template match="xhtml:a[contains(@href,'#') 
                      and not(boolean(ancestor::xhtml:p|ancestor::xhtml:li))]" 
                priority="1.1">
    <p>
      <xref>
        <xsl:attribute name="linkend">
         <xsl:call-template name="make_id">
           <xsl:with-param name="string" select="normalize-space(@href)"/>
         </xsl:call-template>
        </xsl:attribute>
        <xsl:apply-templates select="@*"/>
        <xsl:apply-templates select="." mode="checkHeadings"/>
      </xref>
   </p>
  </xsl:template>
  
  <!-- Table conversion -->
  
  <xsl:template match="xhtml:table">
      <table>
        <xsl:apply-templates select="@*"/>
        <xsl:if test="xhtml:caption">
          <title>
            <xsl:apply-templates select="xhtml:caption/node()"/>
          </title>
        </xsl:if>
        <tgroup>
          <xsl:variable name="columnCount">
            <xsl:for-each select="xhtml:tr | xhtml:tbody/xhtml:tr | xhtml:thead/xhtml:tr">
              <xsl:sort select="count(xhtml:td | xhtml:th)" data-type="number" order="descending"/>
              <xsl:if test="position()=1">
                <xsl:value-of select="count(xhtml:td | xhtml:th)"/>
              </xsl:if>
            </xsl:for-each>
          </xsl:variable>
          <xsl:attribute name="cols">
            <xsl:value-of select="$columnCount"/>
          </xsl:attribute>
          <xsl:if test="xhtml:tr/xhtml:td/@rowspan 
                            | xhtml:tr/xhtml:td/@colspan
                            | xhtml:tbody/xhtml:tr/xhtml:td/@rowspan 
                            | xhtml:tbody/xhtml:tr/xhtml:td/@colspan
                            | xhtml:thead/xhtml:tr/xhtml:th/@rowspan 
                            | xhtml:thead/xhtml:tr/xhtml:th/@colspan">
            <xsl:call-template name="generateColspecs">
              <xsl:with-param name="count" select="number($columnCount)"/>
            </xsl:call-template>
          </xsl:if>
          <xsl:apply-templates select="xhtml:thead"/>
          <tbody>
            <xsl:apply-templates select="xhtml:tr | xhtml:tbody/xhtml:tr | text() | xhtml:b | xhtml:strong | xhtml:i | xhtml:em | xhtml:u, xhtml:tfoot/xhtml:tr"/>
          </tbody>
        </tgroup>
      </table>
  </xsl:template>
  
  <xsl:template match="xhtml:thead">
    <thead>
      <xsl:apply-templates select="." mode="checkHeadings"/>
    </thead>
  </xsl:template>
  
  <xsl:template match="xhtml:tr">
    <row>
      <xsl:apply-templates select="." mode="checkHeadings"/>
    </row>
  </xsl:template>
  
  <xsl:template match="xhtml:th | xhtml:td">
    <xsl:variable name="position" select="count(preceding-sibling::*) + 1"/>
    <entry>
        <xsl:if test="number(@colspan) and @colspan > 1">
          <xsl:attribute name="namest">
            <xsl:value-of select="concat('col', $position)"/>
          </xsl:attribute>
          <xsl:attribute name="nameend">
            <xsl:value-of select="concat('col', $position + number(@colspan) - 1)"/>
          </xsl:attribute>
        </xsl:if>
        <xsl:if test="number(@rowspan) and @rowspan > 1">
          <xsl:attribute name="morerows">
            <xsl:value-of select="number(@rowspan) - 1"/>
          </xsl:attribute>
        </xsl:if>
      <xsl:apply-templates select="." mode="checkHeadings"/>
    </entry>
  </xsl:template>
  

  <xsl:template name="generateColspecs">
    <xsl:param name="count" select="0"/>
    <xsl:param name="number" select="1"/>
    <xsl:choose>
      <xsl:when test="$count &lt; $number"/>
      <xsl:otherwise>
        <colspec>
          <xsl:attribute name="colnum">
            <xsl:value-of select="$number"/>
          </xsl:attribute>
          <xsl:attribute name="colname">
            <xsl:value-of select="concat('col', $number)"/>
          </xsl:attribute>
        </colspec>
        <xsl:call-template name="generateColspecs">
          <xsl:with-param name="count" select="$count"/>
          <xsl:with-param name="number" select="$number + 1"/>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  

  <xsl:function name="oxygen:countCells" as="item()+">
    <xsl:param name="cell" as="element()"/>
    <xsl:param name="count" as="item()"/>
    
    <xsl:variable name="newCount">
      <xsl:choose>
        <xsl:when test="number($cell/@colspan) and $cell/@colspan > 1">
          <xsl:value-of select="number($cell/@colspan) + number($count)"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="1 + number($count)"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:choose>
      <xsl:when test="count($cell/following-sibling::*) > 0">
        <xsl:value-of select="oxygen:countCells($cell/following-sibling::*[1], $newCount)"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$newCount"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>  
  
  <xsl:template match="*" mode="preprocess">
    <xsl:apply-templates select="."/>
  </xsl:template>

  <xsl:template match="*">
    <xsl:apply-templates select="." mode="checkHeadings"/>
  </xsl:template>
  
  <xsl:template match="xhtml:h1 | xhtml:h2 | xhtml:h3 | xhtml:h4 | xhtml:h5 | xhtml:h6" mode="preprocess">
    <p>
      <b>
        <xsl:value-of select="."/>
      </b>
    </p>
  </xsl:template>
</xsl:stylesheet>